breed [families family]
breed [superagents superagent]

globals
[
  experiment-name
  IUE
  parameters-list
  tick-list

  season                         ; hot or cold
  weights-vector                 ; [0.5 0.25 0.12 0.06 0.03 0.015 0.0075 0.00375 0.001875 0.0009375]
  component-index                ; counter of groups

  ;OUTPUT VARIABLES
  number-of-cooperative-units
  number-of-agents-in-cooperative-units
  number-of-isolated-agents
  number-of-living-agents-at-the-beginning      ;(at the beginning of the tick)
  number-of-living-agents-at-the-end            ;(at the end of the tick)
  number-of-labor-units-at-the-beginning        ;(at the beginning of the tick)
  number-of-labor-units-at-the-end              ;(at the end of the tick)
  number-of-family-deaths-this-tick
  number-of-labor-units-deaths-this-tick
  total-number-of-family-deaths
  total-number-of-labor-units-deaths
  total-collected-energy
  collected-energy-standard-deviation
  average-similarity-threshold-in-cooperative-units
  sd-similarity-threshold-in-cooperative-units
  largest-group-size
  number-of-movements
  mean-labor-per-family
  stdev-labor-per-family
  mean-technology-of-families
  std-technology-of-families
  collected-energy-per-labor
  patches-max-resource-stdev
  mean-similarity-threshold
  stdev-similarity-threshold
  ;&&
  social-fractionalization ;; E(p), ELF
  cultural-polarization ;; RQ
  distance-matrix
  similarity-matrix
  depth-of-differences
  generalized-fractionalization ;; G(S)
  spatial-matrix
  spatial-index
  aggregation-index
    aggregation-index2
  adjusted-aggregation-index
  occupation
  average-occupation

]

families-own
[
  my-neighborhood       ; list of families around
  my-helpers            ; subset of my-neighborhood that will help me (with probability 95%)
  my-group              ; list of agents withins a social aggregate (i.e. families that belong to my group/component)
  collected-energy      ; energy obtained during the current tick
  total-energy          ; total level of energy (including surplus)
  identity              ; 10-dimension vector with values 1 (no important) to 6 (very important)
  labor                 ; number of individuals within a household
  survival-threshold    ; 730 * labor
  technology            ; Gaussian-distributed: mean: average-technology; standard deviation: diversity
  storing-factor        ; this parameter is used to calculate the ramaining surplus from the amount of resurces collected in the previous tick
  individual-capability ; fraction of the resource that the agent is able to obtain when working on their own
  similarity-threshold  ; cooperation will be posible if similarity >= similarity-threshold
  cooperation?          ; true if I have been involved in a cooperation process (i.e. I have helped someone or someone has helped me)
  explored?             ; needed for detecting components
  component             ; group (i.e. component in the network) where the family belongs
  my-superagent         ; superagent to which I belong (if any), 0 otherwise.
  move?                 ; takes the value true if this family will move in the beginning of the next tick
  spare-labor           ; number of extra labor units that a family can lend to help other families
  extra-labors-available? ; true if spare-labor > 0 and the family has not offered her help yet
]


superagents-own
[
  members
  list-of-members
  max-number-of-members
  age
  group-consensual-identity
  initial-tick
  final-tick
  dead?
]


patches-own
[
  difficulty-cold-season  ; (h)
  difficulty-hot-season   ; (h)
  max-resource     ; maximum amount of resource (in hot season)
  resource         ; current amount of resource
  amount-consumed  ; amount of energy consumed during the current tick
  exploited?       ; true during one tick after been exploited
  number-of-superagents-per-patch  ; spatial segregation: number of superagents whose influence area is over me
]



to setup
  clear-all
  reset-ticks

  ;OUTPUT FILES
  initialize-output-files
  set parameters-list (list "cooperation-allowed?" "help-limited-to-spare-labor?" "initial-population" "mean-resource-on-patches" "sd-resource-on-patches""average-technology" "diversity" "labor-average" "average-storing-factor" "movement" "internal-change-rate")
  write-parameters-file

  ;set tick-list [48 49 98 99 248 249 498 499]
  set tick-list [500]

  ;SET GLOBAL VARIABLES
  set weights-vector [0.5 0.25 0.12 0.06 0.03 0.015 0.0075 0.00375 0.001875 0.0009375]


  ;CREATE AND INITIALIZE FAMILIES

  let initial-identity n-values 10 [1 + random 5]  ;all the families start with the same identity vector

  ask n-of initial-population patches
  [
    sprout-families 1
    [
      set shape "person"
      set color red
      set identity initial-identity
      set labor random-poisson labor-average if labor < 2 [set labor 2] ;(correction for cases with labor < 2)
      set survival-threshold (730 * labor) / 2
      set technology random-normal average-technology diversity
          ; correction to force technology to be in the interval [0 2]
          if technology < 0.05 [set technology 0.05]
          if technology > 2 [set technology 2]


      set storing-factor random-normal average-storing-factor diversity
      ; correction to force storing-factor to be in the interval [0 1]
      if storing-factor < 0.01 [set storing-factor 0.01]
      if storing-factor > 0.91 [set storing-factor 0.91]

      set total-energy 1 + survival-threshold * (1 + storing-factor) / storing-factor    ; this means that all the families will have enough energy to survive the first tick without hunting
      set collected-energy 0
      set my-group families with [who < 0]     ; initialize my-group = nobody
      set move? false
    ]
  ]


  ;INITIALIZE PATCHES
  ask patches
  [
    ; Resources are distributed in a Gaussian distribution with mean mean-resource-on-patches and standard deviation 1000
    ;set max-resource (max-resource-on-patches / 1000) + random ( max-resource-on-patches - (max-resource-on-patches / 1000) )
    set max-resource random-normal mean-resource-on-patches sd-resource-on-patches if max-resource < 0 [set max-resource 0]
    set difficulty-cold-season random-normal 0.6 0.3 if difficulty-cold-season < 0.01 [set difficulty-cold-season 0.01]
    set difficulty-hot-season random-normal 0.3 0.1 if difficulty-hot-season < 0.01 [set difficulty-hot-season 0.01]
    set exploited? false
    set amount-consumed 0
    set pcolor 69
  ]
  set patches-max-resource-stdev standard-deviation [max-resource] of patches
  set-default-shape superagents "circle"
end





to go
  ask families with [move?] [move set move? false]
  ask families [set color red]

  survive
  update-resources-on-patches
  update-similarity-threshold
  hunt-and-gather
  identify-groups
  update-identity
  update-technology
  update-storing-factor
  decide-whether-to-move-or-stay
  write-agents-file
  write-superagents-file
  ask superagents with [dead?] [die]
  update-output-variables
  write-output-file
  if print-spatial-matrix? [write-spatial-matrix]
  tick
  ;if ticks = 500 or not any? families [stop]
  if ticks = 700 [stop]
end





to survive
  set number-of-labor-units-deaths-this-tick 0
  set number-of-family-deaths-this-tick 0
  set number-of-living-agents-at-the-beginning count families
  set number-of-labor-units-at-the-beginning sum [labor] of families

  ; 1. Substract survival-threshold. If they didn't get enough energy to reach their survival threshold, some labor units die
  ask families
    [
      let max-maintainable-labors floor (total-energy / 365) ; max number of labor units within the family that can survive with the amount of energy collected

      if max-maintainable-labors < labor ; I can't maintain all my labor units
      [
        set number-of-labor-units-deaths-this-tick number-of-labor-units-deaths-this-tick + (labor - max-maintainable-labors)
        set labor max-maintainable-labors
        set survival-threshold 365 * labor
      ]

      set total-energy total-energy - survival-threshold

      if labor < 2      ;if the amount of labor goes below 2, the family (agent) dies
      [
        set number-of-family-deaths-this-tick number-of-family-deaths-this-tick + 1
        set number-of-labor-units-deaths-this-tick number-of-labor-units-deaths-this-tick + labor
        die
      ]

    ]


  ; 2. Vegatative reproduction: The amount of labor within a family goes up one unit every 30 ticks
  ask families
  [
    if ticks > 1 and remainder ticks 30 = 0
    [
      set labor labor + 1
      set survival-threshold (730 * labor) / 2
    ]
  ]

  ; if the amount of labor reaches the value 10, the family is split into two families with probability 95%
  ; (provided there is enough room in the neighbourhood for another family: there cannot be more than one family in the same patch)
  ask families
  [
    if labor >= 10
    [
      if random-float 1 < .95
      [
        if any? patches in-radius movement with [not any? turtles-here]
          [
            set labor floor (labor / 2)
            set survival-threshold (730 * labor) / 2
            hatch-families 1 [move-to one-of patches in-radius movement with [not any? turtles-here]]
          ]
      ]
    ]
  ]

  ;ask superagents with [count members <= 1] [die]
  ask superagents with [count members <= 1] [set final-tick ticks set dead? true]


end





to update-resources-on-patches
  set season ifelse-value (remainder ticks  2 = 0) ["hot"] ["cold"]
  ask patches
  [
    ifelse exploited?
    [
      ; patches that were exploited in the previous tick:
      ; if the current season is cold, the patch will not regenerate completely: (max-resource - amount-consumed)/2
      ; if the current season is hot, the level of resource will be max-resource (no matter whether it was exploited during the previous tick or not)
      if season = "hot" [set resource max-resource]
      if season = "cold" [set resource (max-resource - amount-consumed) / 2]
    ]
    [
      set resource ifelse-value (season = "hot") [max-resource] [max-resource / 2]
    ]
    set exploited? false
    set amount-consumed 0
  ]
  ask patches [set plabel ""]
  ifelse (display-resorces?) [ask patches [set pcolor scale-color green resource 100000 100]] [ask patches [set pcolor 69]]
end





to update-similarity-threshold
  ask families
  [
    set spare-labor 0
    set extra-labors-available? false
    set individual-capability calculate-individual-capability



    ifelse ([resource] of patch-here) <= survival-threshold
    [
      ; They will refuse to cooperate, as they are unable to reach their survival-threshold no matter the number of families who help them
      set similarity-threshold 1
    ]
    [
      let difficulty ifelse-value (season = "cold") [[difficulty-cold-season] of patch-here] [[difficulty-hot-season] of patch-here]
      let extra-workers (survival-threshold / (difficulty * (([resource] of patch-here) - survival-threshold) ) ) ^ ( 1 / technology) - labor
      ifelse extra-workers < 0
      [
        ; I have more labor units than necessary
        set spare-labor floor ((-1) * extra-workers)
        set extra-workers 0           ; (I will use this value later to calculate the similarity threshold: ST = extra-workes / 100 (0 in this case).
        set extra-labors-available? true
      ]
      [
        ; I don't have enough labor units to reach my survival thereshold (and therefore I can't help other families)
        set spare-labor 0
        set extra-labors-available? false
      ]

      set similarity-threshold min list 1 ((1 / 100) * extra-workers) ; similarity-threshold could be greater than 1 if the number of extra workers goes above 100 --> --> in this case similarity-threshold = 1
    ]
  ]
end




to hunt-and-gather
  ask families
  [
    set total-energy total-energy * storing-factor ;depreciate agents' energy at the beginning of the tick
    set cooperation? false
    set my-helpers families with [who < 0] ; (empty agentlist)
    set my-group families with [who < 0]   ; (empty agentlist)
    set collected-energy 0
  ]

  ask links [die]


  ; families with total-energy > survival-threshold don't hunt or gather

  ask families with [total-energy < survival-threshold]
  [
    ; Try to act individually:

    let productivity ([resource] of patch-here) * individual-capability

    ifelse cooperation-allowed?
    [


      ; FAMILIES WILL ASK FOR HELP ONLY IF THEY ARE UNABLE TO REACH THEIR SURVIVAL THRESHOLD
      ifelse total-energy + productivity >= survival-threshold
      [
        set color green

        ; ACT INDIVIDUALLY: I don't need any help: I will act individually and collect as much energy as I need to reach my survival-threshold

        let energy-needed survival-threshold - total-energy


        ifelse energy-needed > [resource] of patch-here
        [
          ; this patch will not give me as much food as i need, but I will take as much as i can from it
          set collected-energy productivity
        ]
        [
          ; this patch has more food than i need (which does not necessarily mean i can obtain it all). Even if I could, I would only take as much as I need to reach my survival threshold
          ifelse productivity >  energy-needed
          [
            ; I could take more than I need, but still I take just what I need
            set collected-energy energy-needed
          ]
          [; I can't get as much as I need, but I take as much as I can
            set collected-energy productivity
          ]

        ]


        set total-energy total-energy + collected-energy
        ask patch-here ;update resource on patch:
          [
            set amount-consumed [collected-energy] of myself
            set resource resource - amount-consumed
            set exploited? true
          ]
      ]

      [
        ; COOPERATION (I need to ask for help)

        set color yellow

        let capability individual-capability ;individual-capability has been calculated in the procedure update-similarity-threshold

        set my-neighborhood other families-on patches in-radius movement
        let potential-helpers my-neighborhood with [(get-similarity identity ([identity] of myself)) > similarity-threshold ] ; similarity-threshold of my neighbors'
        set my-helpers ask-for-help potential-helpers

        if any? my-helpers; if someone helps me, my capability will be aggregated-capability. Otherwise it will be my individual-capability
        [
          set capability calculate-aggregated-capability my-helpers
          ask my-helpers [create-link-with myself [set color black set thickness .1]]   ;helpers create a link with helped agents
          set cooperation? true ; I have cooperated...
          ask my-helpers [set cooperation? true]  ; ... and so have my helpers

          if help-limited-to-spare-labor? [ask my-helpers[set extra-labors-available? false]]
        ]

        set collected-energy ([resource] of patch-here) * capability

        set total-energy total-energy + collected-energy        ;(therefore, total-energy might be greater than her survival-threshold


        ;update resource on patch:
        ask patch-here
          [
            set amount-consumed [collected-energy] of myself
            set resource resource - amount-consumed
            set exploited? true
          ]
      ]
    ]



    [
      ; COOPERATION NOT ALLOWED: FAMILIES ARE NOT ALLOWED TO ASK FOR HELP IF THEY ARE UNABLE TO REACH THEIR SURVIVAL THRESHOLD. THEY WILL ALWAYS ACT INDIVIDUALLY


      let energy-needed survival-threshold - total-energy
      ifelse energy-needed > [resource] of patch-here
        [
          ; this patch will not give me as much food as i need, but I will take as much as i can from it
          set collected-energy productivity
        ]
        [
          ; this patch has more food than i need (which does not necessarily mean i can obtain it all). Even if I could, I would only take as much as I need to reach my survival threshold
          ifelse productivity >  energy-needed
          [
            ; I could take more than I need, but still I take just what I need
            set collected-energy energy-needed
          ]
          [; I can't get as much as I need, but I take as much as I can
            set collected-energy productivity
          ]

        ]

      set total-energy total-energy + collected-energy
      ask patch-here ;update resource on patch:
        [
          set amount-consumed [collected-energy] of myself
          set resource resource - amount-consumed
          set exploited? true
        ]
    ]
  ]
end





to-report calculate-individual-capability
  let difficulty ifelse-value (season = "cold") [[difficulty-cold-season] of patch-here] [[difficulty-hot-season] of patch-here]
  report 1 / (1 + (1 / (difficulty * (labor ^ technology))))
end





to-report ask-for-help [potential-helpers]
  ; Determine the list of agents that will cooperate
  let agents-willing-to-cooperate []
  if count potential-helpers > 0  ; if my list of helpers is not empty
  [
    ;each one of my helpers will help me with p=95%
    foreach [self] of potential-helpers
    [
      if random-float 1 < 1 / (1 + e ^((5 / movement) * (distance ? - movement))) ;;;;; Acceptation inversely proportional to  distance
      ;if random-float 1 < 0.95
      [
        set agents-willing-to-cooperate fput ? agents-willing-to-cooperate
      ]
    ]
  ]

  set agents-willing-to-cooperate turtle-set agents-willing-to-cooperate
  if help-limited-to-spare-labor? [set agents-willing-to-cooperate agents-willing-to-cooperate with [extra-labors-available?]]

  report agents-willing-to-cooperate
end




to-report calculate-aggregated-capability [agents-willing-to-cooperate]
  let max-technology-within-my-local-group max [technology] of (turtle-set agents-willing-to-cooperate self)   ; maximum technology within my group (including myself)
  let x [resource] of patch-here / (mean-resource-on-patches + 3 * sd-resource-on-patches)
  if x > 1 [set x 1]
  if x < 0 [set x 0]
  let returns-to-cooperation 2 - x
  let total-labor 0
  ifelse help-limited-to-spare-labor?
  [
    set total-labor labor + sum [spare-labor] of turtle-set agents-willing-to-cooperate ; only spare labor units within my group (including myself)
  ]
  [
    set total-labor labor + sum [labor] of turtle-set agents-willing-to-cooperate ; all of the labor units within my group (including myself)
  ]
  let difficulty ifelse-value (season = "cold") [[difficulty-cold-season] of patch-here] [[difficulty-hot-season] of patch-here]
  let aggregated-capability 1 / (1 + ( 1 / ( ( difficulty * (total-labor ^ max-technology-within-my-local-group)) ^ returns-to-cooperation ) ) )
  report aggregated-capability
end




to-report get-similarity [a b]
  let ap (map [?1 * ?2] a weights-vector)
  let bp (map [?1 * ?2] b weights-vector)
  let numerator sum (map [?1 * ?2] ap bp)
  let denominator sqrt((sum map [? * ?] ap)) * sqrt((sum map [? * ?] bp))
  report numerator / denominator
end





to identify-groups
  find-all-components
  update-superagents

  ask families with [component > 0]
  [
    set my-group families with [component = [component] of myself]
  ]
end


;; The following two procedures are an adaptation from: Wilensky, U. (2005). NetLogo Giant Component model.
;; http://ccl.northwestern.edu/netlogo/models/GiantComponent.
;; Center for Connected Learning and Computer-Based Modeling, Northwestern University, Evanston, IL.
to find-all-components
  ask families [set explored? false]
  ask families with [not any? link-neighbors] [set component 0 set explored? true] ; families that don't cooperate (isolated agents) will have component = 0
  set component-index 0
  loop
  [ ifelse any? families with [not explored?]
      [
        let start one-of sort families with [ not explored? ]
        set component-index ( component-index + 1)
        ask start [ explore ]
      ]
      [stop]
  ]
end

;; Finds all families reachable from this node
to explore
  if explored? [stop]
  set explored? true
  set component component-index
  ask link-neighbors [explore]
end





to decide-whether-to-move-or-stay

  ask families
  [
    ; 1. Calculate if I will be able to survive without working one more tick with my current level of energy (bearing in mind the depreciation factor) .
    ; If that is the case, I leave with probability 0.05
    ifelse (total-energy - survival-threshold) * storing-factor > survival-threshold
    [
      if (random-float 1 < 0.05) [set move? true]
    ]
    [
      ; 2. I will have to work next tick because I will not be able to get by with my (depreciated) level of energy
      ; Before moving, I will check if I will be able to get enough resources if I stay in the same patch
      let resources-next-tick ifelse-value (season = "hot") [([max-resource] of patch-here - collected-energy) / 2] [[max-resource] of patch-here]

      ifelse (total-energy - survival-threshold) * storing-factor + resources-next-tick * individual-capability  > survival-threshold
      [
        ; If I can survive here, I will probably stay, but I will leave with probability 0.05
        if (random-float 1 < 0.05) [set move? true]
      ]
      [
        set move? true
      ]
    ]
  ]
end




to move
  if any? patches in-radius movement with [not any? turtles-here]
  [
    move-to one-of patches in-radius movement with [not any? turtles-here]
  ]
end





to update-identity

  ; 1. Diffusion process (only for agents that have cooperated)
  if any? superagents
  [
    ask superagents
    [
      let consensual-identity compute-consensual-identity members
      ask members
      [
        if random-float 1 < 0.95 [set identity consensual-identity]
      ]
      set group-consensual-identity consensual-identity
    ]
  ]




  ; 2. Mutation process (for all families)
  ask families
  [
    if random-float 1 < internal-change-rate
    [
      let index-vector n-values 10 [?]
      foreach index-vector
        [
          if random-float 1 >= (item ? weights-vector) [set identity replace-item ? identity (1 + random 5)]
        ]
    ]
  ]
end





to-report compute-consensual-identity [group]
  let consensus []
  foreach n-values 10 [?]
    [
      let consensual-trait one-of modes [ item ? identity] of group
      set consensus fput consensual-trait consensus
    ]
  set consensus reverse consensus
  report consensus
end





to update-technology


  ; 1. Diffusion process (only for agents that have cooperated)
  if any? superagents
  [
    ask superagents
    [
      let average-technology-in-this-superagent mean [technology] of members
      let max-technology-in-this-superagent max [technology] of members

      ask members
      [
        ifelse technology < 0.95 * average-technology-in-this-superagent
        [
          if random-float 1 < 0.95
          [
            set technology technology + 0.1
            if technology > max-technology-in-this-superagent [set technology max-technology-in-this-superagent]
          ]
        ]
        [
          if technology < 1.05 * average-technology-in-this-superagent
            [
              if random-float 1 < 0.95
              [
                set technology technology + 0.01
                if technology > max-technology-in-this-superagent [set technology max-technology-in-this-superagent]
              ]
            ]
        ]
      ]
    ]
  ]



  ; 2. Mutation process (for all families)
  ask families
  [
    if random-float 1 < internal-change-rate
    [
      set technology random-normal average-technology diversity
      ; possible correction to force technology to be in the interval [0 2] (and thus depreciation will be in [0 1]
      if technology < 0.05 [set technology 0.05]
      if technology > 2 [set technology 2]
    ]
  ]
end



to update-storing-factor


  ; 1. Diffusion process (only for agents that have cooperated)
  if any? superagents
  [
    ask superagents
    [
      let average-storing-factor-in-this-superagent mean [storing-factor] of members
      let max-storing-factor-in-this-superagent max [storing-factor] of members

      ask members
      [
        ifelse storing-factor < 0.95 * average-storing-factor-in-this-superagent
        [
          if random-float 1 < 0.95
          [
            set storing-factor storing-factor + 0.1
            if storing-factor > max-storing-factor-in-this-superagent [set storing-factor max-storing-factor-in-this-superagent]
          ]
        ]
        [
          if storing-factor < 1.05 * average-storing-factor-in-this-superagent
            [
              if random-float 1 < 0.95
              [
                set storing-factor storing-factor + 0.01
                if storing-factor > max-storing-factor-in-this-superagent [set storing-factor max-storing-factor-in-this-superagent]
              ]
            ]
        ]
      ]
    ]
  ]



  ; 2. Mutation process (for all families)
  ask families
  [
    if random-float 1 < internal-change-rate
    [
      set storing-factor random-normal average-storing-factor diversity
      ; possible correction to force storing-factor to be in the interval [0 1]
      if storing-factor < 0.01 [set storing-factor 0.01]
      if storing-factor > 0.91 [set storing-factor 0.91]
    ]
  ]
end





to update-superagents

  ask families [set my-superagent 0]

  ifelse count superagents = 0
  [
    ; if there are no super-agents, all the identified components become super-agents
    foreach n-values component-index [? + 1]
    [
      create-superagents 1 [set members families with [component = ?] set age 0 set initial-tick ticks set dead? false]
    ]
  ]

  [
    ; if there exists any super-agents, two actions may take place: update their list of families or die

    let components-to-analyze n-values component-index [? + 1]

    ask superagents
    [
      let previous-members [self] of members
      let current-location-of-previous-members map [[component] of ?] previous-members


      ;IF AT LEAST 60% OF THE MEMBERS BELONG TO THE SAME COMPONENT, THE SUPER-AGENT STILL EXISTS
      let target-component-index one-of modes current-location-of-previous-members



      ifelse length filter [? = target-component-index] current-location-of-previous-members >= floor 0.6 * length previous-members and target-component-index > 0 ; because component 0 implies isolated agents
      [
        ;update list of members
        set members families with [component = target-component-index]
        set components-to-analyze remove target-component-index components-to-analyze
        set age age + 1
      ]

      ; IF NOT, THE SUPER-AGENT DIES
      [
        set final-tick ticks
        set dead? true
        ;die
      ]
    ]

    ; THE LIST components-to-analyze contents the components that are new super-agents
    foreach components-to-analyze
    [
      create-superagents 1
      [
        set members families with [component = ?]
        set age 0
        set initial-tick ticks
        set dead? false
      ]
    ]

  ]



  ask superagents [move-to one-of members with-min [who] set label-color black set size 0 set label (word who "/" age)]

  ; This happens as a consequence of a fusion
  ask patches with [count superagents-here > 1]
    [
      ;we will keep the oldest superagent
      let coincident-super-agents sort-on [age] superagents-here  ;the super-agent in the last position of this list survives
      let super-agents-to-kill but-last coincident-super-agents
      ask turtle-set super-agents-to-kill
      [
        set final-tick ticks
        set dead? true
        ;die
      ]
    ]


  ;Update max-number-of-members
  ask superagents
    [
      if count members > max-number-of-members [set max-number-of-members count members]
    ]


  ask superagents [set list-of-members [self] of members]
  ask superagents [ask members [set my-superagent [who] of myself]]

end




to update-output-variables

  set number-of-living-agents-at-the-end count families
  set number-of-labor-units-at-the-end sum [labor] of families
  set number-of-cooperative-units count superagents
  set number-of-agents-in-cooperative-units sum [count members] of superagents
  set number-of-isolated-agents count families with [not cooperation?]
  set largest-group-size ifelse-value any? superagents [first [count members] of superagents with-max [count members]] [0]

  set total-collected-energy ifelse-value (any? families) [sum [collected-energy] of families] [0]
  set collected-energy-standard-deviation ifelse-value (count families >= 2) [standard-deviation [collected-energy] of families] [0]
  set average-similarity-threshold-in-cooperative-units ifelse-value any? families with [cooperation?] [mean [similarity-threshold] of families with [cooperation?]] [0]
  set sd-similarity-threshold-in-cooperative-units ifelse-value any? families with [cooperation?] [standard-deviation [similarity-threshold] of families with [cooperation?]] [0]



  set mean-similarity-threshold ifelse-value (count families > 0) [mean [similarity-threshold] of families] [0]
  set stdev-similarity-threshold ifelse-value (count families >= 2) [standard-deviation [similarity-threshold] of families] [0]


  set number-of-movements count families with [move?]

  set total-number-of-family-deaths total-number-of-family-deaths + number-of-family-deaths-this-tick
  set total-number-of-labor-units-deaths total-number-of-labor-units-deaths + number-of-labor-units-deaths-this-tick

  if any? families
  [
    set mean-technology-of-families mean [technology] of families
    set mean-labor-per-family mean [labor] of families
    set stdev-labor-per-family ifelse-value (count families >= 2) [standard-deviation [labor] of families] [0]
    set collected-energy-per-labor total-collected-energy / number-of-labor-units-at-the-end
  ]
  if count families >= 2
  [
    set std-technology-of-families  standard-deviation [technology] of families
  ]

  ;&&
  ;Calculate GROUP FRACTIONALIZATION (ELF)
  ifelse count superagents = 0
  [
    ifelse (any? families) [set social-fractionalization 1] [set social-fractionalization "NaN"]
  ]
  [
    let p-squared 0
    foreach [self] of superagents
    [
      set p-squared p-squared + ((count [members] of ?) / (count families))^ 2
    ]
    set social-fractionalization 1 - (1 / (count superagents)^ 2) * p-squared
  ]

  ;&&
  ;Calculate DEPTH OF DIFFERENCES AND GENERELIZED FRACTIONALIZATION G(S)
  ifelse any? families
  [
    ;Depth of differences
    set distance-matrix build-distance-matrix ;; The element i,j of the distance matrix represents the (non-normalized) euclidean distance between family i's identity vector and family j's identity vector
    set depth-of-differences max distance-matrix - min distance-matrix

    ;Generalized fractionalization
    set similarity-matrix build-similarity-matrix distance-matrix ;the element i,j of the similarity matrix is 1 if family i and family j are identical, and 0 if family i and family j are totally different
    set generalized-fractionalization 1 - (1 / count families)^ 2 * sum similarity-matrix
  ]
  [
    set depth-of-differences "NaN"
    set generalized-fractionalization "NaN"
  ]


  ;Calculate CULTURAL FRACTIONALIZATION (RQ)
  ifelse count superagents = 0
  [
    ifelse any? families [set cultural-polarization 1] [set cultural-polarization "NaN"]
  ]
  [
    let p 0
    foreach [self] of superagents
      [
        let pk (count [members] of ?) / (count families)
        set p p + (((0.5 - pk) / 0.5)^ 2) * pk
        ;show p
      ]
    set cultural-polarization 1 - p
  ]



  ;Calculate SUPERAGENT'S INFLUENCE AREA
  ask patches
  [
    set number-of-superagents-per-patch 0
  ]
  ask superagents
  [
    let minimum-xcor [xcor] of min-one-of members [xcor]
    let maximum-xcor [xcor] of max-one-of members [xcor]
    let minimum-ycor [ycor] of min-one-of members [ycor]
    let maximum-ycor [ycor] of max-one-of members [ycor]
    let superagent-influence-area patches with [pxcor >= minimum-xcor and pxcor <= maximum-xcor and pycor >= minimum-ycor and pycor <= maximum-ycor]
    let my-color one-of [5 15 25 35 45 55 65 75 85  95 105 115 125 135 7 17 27 37 47 57 67 77 87 97 107 117 127 137]
    ask superagent-influence-area
    [
      set number-of-superagents-per-patch number-of-superagents-per-patch + 1
      if display-influence-area? [set pcolor my-color]
    ]
  ]

  if display-influence-area? [ask patches [set plabel-color 0 set plabel (precision number-of-superagents-per-patch 1)]]


  set spatial-matrix []  ;; this variable will be a list of lists (matrix) that will contain the variable "superagents-per-patch" stored in every patch
  let sorted-patches-list sort patches   ;; this list contains a sorted list of patches, i.e. from patch 0 39 (top left corner) to patch 79 0 (bottom right corner)
  let data map [[number-of-superagents-per-patch] of ?] sorted-patches-list  ;this list contains the data to be introduced into the matrix
  let row-index reverse n-values (max-pycor + 1) [?] ; [39 38 37 36 ... 2 1 0]
  foreach row-index
    [
      set spatial-matrix fput (sublist data 0 80) spatial-matrix ; introduce lists of 80 elements of data into the spatial-matrix
      set data sublist data 80 (length data) ;remove data already copied into the matrix
    ]
  set spatial-matrix reverse spatial-matrix
  ;show spatial-matrix

  set spatial-index ifelse-value any? superagents [(sum [number-of-superagents-per-patch] of patches) / count superagents] ["NaN"]

  ;; Calculate aggregation index;;
  set occupation 0
  ask patches [set occupation occupation + number-of-superagents-per-patch]
  set average-occupation ifelse-value any? superagents [occupation / count patches with [number-of-superagents-per-patch > 0]] [0]
  set aggregation-index ifelse-value any? superagents [average-occupation / number-of-cooperative-units] [0]
  set adjusted-aggregation-index ifelse-value (count superagents > 1) [(average-occupation - 1) / (number-of-cooperative-units - 1)] [0]
end


;(1) Para crear el nombre del experimento para los archivos:

to initialize-output-files
  set experiment-name (word (remove ":" date-and-time) "." random 100 ) ;; es una variable global que se inicializa en cada run
  set IUE ( word ( substring (remove "." (remove ":" date-and-time)) 0 9 ) random 10 )
end



;(2) Para escribir los parámetros
to write-parameters-file
  file-open (word experiment-name "_parameter.txt")  ;; llamo al fichero correspondiente
  file-print (word "IUE: " IUE)
  foreach parameters-list ;;tengo los nombres de los parámetros en una lista
      [ file-print (word ? " " (runresult ?)) ] ;; escribo nombre y valor separado por un espacio
  file-close
end



;(3) Para escribir en cada tick variables agregadas del sistema:

to write-output-file
  file-open (word experiment-name "_output.txt") ;; llamo al fichero correspondiente
  ;file-print (word ticks " " number-of-living-agents-at-the-beginning " " number-of-living-agents-at-the-end " " number-of-family-deaths-this-tick " " total-number-of-family-deaths " " number-of-labor-units-at-the-beginning " " number-of-labor-units-at-the-end " " number-of-labor-units-deaths-this-tick " " total-number-of-labor-units-deaths " " mean-labor-per-family " " stdev-labor-per-family " " mean-technology-of-families " " std-technology-of-families " " mean-similarity-threshold " " stdev-similarity-threshold " " number-of-movements " " number-of-agents-in-cooperative-units " " number-of-isolated-agents " " number-of-cooperative-units " " largest-group-size " " average-similarity-threshold-in-cooperative-units " " sd-similarity-threshold-in-cooperative-units " " patches-max-resource-stdev " " total-collected-energy " " collected-energy-standard-deviation " " collected-energy-per-labor)
  file-print (word IUE " " ticks " " number-of-family-deaths-this-tick " " number-of-labor-units-deaths-this-tick " " number-of-movements " " social-fractionalization " " depth-of-differences " " generalized-fractionalization " "cultural-polarization " " spatial-index)
  file-close
end


to write-agents-file
  file-open (word experiment-name "_agents.txt") ;; llamo al fichero correspondiente
  ask families
  [
    file-print (word IUE " " ticks " " who " " xcor " " ycor " " precision collected-energy 3 " " precision total-energy 3 " " labor " " survival-threshold " " precision technology 3 " " precision storing-factor 3 " " precision individual-capability 3 " " precision similarity-threshold 3 " " cooperation? " " my-superagent " " item 0 identity " " item 1 identity " " item 2 identity " " item 3 identity " " item 4 identity " " item 5 identity " " item 6 identity " " item 7 identity " " item 8 identity " " item 9 identity)
  ]
  file-close
end


to write-superagents-file
  file-open (word experiment-name "_superagents.txt") ;; llamo al fichero correspondiente
  ask superagents
  [

    file-print (word IUE " " ticks " " who " " initial-tick " " final-tick " " max-number-of-members " " item 0 group-consensual-identity " " item 1 group-consensual-identity " " item 2 group-consensual-identity " " item 3 group-consensual-identity " " item 4 group-consensual-identity " " item 5 group-consensual-identity " " item 6 group-consensual-identity " " item 7 group-consensual-identity " " item 8 group-consensual-identity " " item 9 group-consensual-identity)
  ]
  file-close
end

to write-spatial-matrix
      file-open (word experiment-name "_spatial-matrix.txt") ;; llamo al fichero correspondiente
      file-print (word IUE " " ticks " " spatial-matrix)
      file-close
end







to-report build-distance-matrix
  ;; The element i,j of the distance matrix represents the (non-normalized) euclidean distance between family i's identity vector and family j's identity vector
  let families-id [who] of families
  let i 0
  let j 0
  let limit length families-id - 1
  let dist-matrix []
  while [i <= limit]
  [
    while [j <= limit]
    [
      ;type  i
      ;type " "
      ;print j
      set dist-matrix fput (calculate-euclidean-distance ([identity] of one-of families with [who = item i families-id]) ([identity] of one-of families with [who = item j families-id])) dist-matrix
      set j j + 1
    ]
    set i i + 1
    set j 0
  ]
  report dist-matrix
end



to-report build-similarity-matrix [dist-matrix]
  let maximum-distance calculate-euclidean-distance [1 1 1 1 1 1 1 1 1 1] [5 5 5 5 5 5 5 5 5 5] ;;the maximum distance is approx. 12.65
  let normalized-distance-matrix map [? / maximum-distance] dist-matrix ; the element i,j of the normalized-distance-matrix is dist(family i's identity, family j's identity) / 12.65 (their elements are bounded between 0 and 1: 0 if identical, 1 if totally different)
  ;show normalized-distance-matrix
  let sim-matrix map [1 - ?] normalized-distance-matrix ;the element i,j of the similarity matrix is 1 if family i and family j are identical, and 0 if family i and family j are totally different
  report sim-matrix
end



to-report calculate-euclidean-distance [vector1 vector2]
let a 0
(foreach vector1 vector2
  [
    set a a + (?2 - ?1)^ 2
  ]
)
let euclidean-distance sqrt (a)
report euclidean-distance
end
@#$#@#$#@
GRAPHICS-WINDOW
11
71
1061
622
-1
-1
13.0
1
10
1
1
1
0
0
0
1
0
79
0
39
0
0
1
ticks
30.0

BUTTON
736
31
840
64
SETUP
setup
NIL
1
T
OBSERVER
NIL
S
NIL
NIL
1

BUTTON
843
31
947
64
GO (one tick)
go
NIL
1
T
OBSERVER
NIL
G
NIL
NIL
1

BUTTON
953
31
1057
64
GO
go
T
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

SLIDER
11
622
180
655
initial-population
initial-population
10
500
150
10
1
NIL
HORIZONTAL

SLIDER
522
621
694
654
labor-average
labor-average
3
8
8
1
1
NIL
HORIZONTAL

SLIDER
352
621
524
654
average-technology
average-technology
0.05
2
1.05
0.1
1
NIL
HORIZONTAL

SLIDER
352
654
524
687
diversity
diversity
0
1
0.1
.1
1
NIL
HORIZONTAL

SLIDER
694
654
866
687
internal-change-rate
internal-change-rate
0
1
0.01
.01
1
NIL
HORIZONTAL

MONITOR
128
26
252
71
SEASON
season
17
1
11

MONITOR
8
26
128
71
CYCLE
max list 0 (ticks - 1)
17
1
11

SWITCH
872
620
1061
653
display-resorces?
display-resorces?
1
1
-1000

MONITOR
1302
451
1543
496
Number of isolated agents
number-of-isolated-agents
17
1
11

MONITOR
1061
79
1302
124
Number of family deaths (current tick)
number-of-family-deaths-this-tick
17
1
11

MONITOR
1061
34
1302
79
Agents at the beginning of the tick
number-of-living-agents-at-the-beginning
17
1
11

MONITOR
1061
659
1302
704
Total collected energy
total-collected-energy
2
1
11

MONITOR
1301
659
1542
704
Standard deviation of collected energy
collected-energy-standard-deviation
2
1
11

MONITOR
1061
540
1304
585
Avg. similarity threshold in cooperative units
average-similarity-threshold-in-cooperative-units
3
1
11

MONITOR
1301
34
1542
79
Agents at the end of the tick
number-of-living-agents-at-the-end
17
1
11

TEXTBOX
1205
431
1430
465
COOPERATIVE UNITS STATISTICS
14
0.0
1

TEXTBOX
1237
12
1387
30
SURVIVAL STATISTICS
14
0.0
1

TEXTBOX
1243
593
1393
611
ENERGY STATISTICS
14
0.0
1

SWITCH
302
31
475
64
cooperation-allowed?
cooperation-allowed?
0
1
-1000

MONITOR
1302
79
1542
124
Number of family deaths (cumulative)
total-number-of-family-deaths
17
1
11

SLIDER
693
621
866
654
movement
movement
1
80
80
1
1
NIL
HORIZONTAL

MONITOR
1061
290
1304
335
Mean technology of families
mean-technology-of-families
2
1
11

MONITOR
1304
290
1542
335
NIL
std-technology-of-families
2
1
11

SLIDER
180
622
352
655
mean-resource-on-patches
mean-resource-on-patches
4000
20000
5000
500
1
NIL
HORIZONTAL

MONITOR
1061
703
1542
748
Collected energy per labor
collected-energy-per-labor
2
1
11

MONITOR
1061
614
1542
659
Std. deviation of max-resource
patches-max-resource-stdev
17
1
11

TEXTBOX
1222
226
1410
260
HOUSEHOLD STATISTICS
14
0.0
1

MONITOR
1061
246
1304
291
Mean labor per family
mean-labor-per-family
2
1
11

MONITOR
1303
246
1542
291
Std.deviation of labor per family
stdev-labor-per-family
2
1
11

MONITOR
1061
335
1304
380
Mean Similarity Threshold
mean-similarity-threshold
2
1
11

MONITOR
1304
335
1542
380
Std. deviation of Similarity Threshold
stdev-similarity-threshold
2
1
11

MONITOR
1061
379
1542
424
Number of movements
number-of-movements
2
1
11

SWITCH
478
31
686
64
help-limited-to-spare-labor?
help-limited-to-spare-labor?
0
1
-1000

MONITOR
1061
451
1303
496
Number of agents in cooperative units
number-of-agents-in-cooperative-units
17
1
11

MONITOR
1061
495
1303
540
Number of cooperative units
number-of-cooperative-units
17
1
11

MONITOR
1303
495
1543
540
Number of agents in the largest cooperative unit
largest-group-size
17
1
11

MONITOR
1061
124
1302
169
Labor units at the beginning of the tick
number-of-labor-units-at-the-beginning
17
1
11

MONITOR
1301
124
1542
169
Number of labor units at the end of the tick
number-of-labor-units-at-the-end
17
1
11

MONITOR
1301
169
1542
214
Number of labor-units deaths (cumulative)
total-number-of-labor-units-deaths
17
1
11

SLIDER
522
654
695
687
average-storing-factor
average-storing-factor
0
1
0.5
0.1
1
NIL
HORIZONTAL

MONITOR
1304
540
1543
585
Std. dev. of similarity thres. in cooperative units
sd-similarity-threshold-in-cooperative-units
2
1
11

SLIDER
181
654
352
687
sd-resource-on-patches
sd-resource-on-patches
1000
1000
1000
0
1
NIL
HORIZONTAL

MONITOR
1060
169
1301
214
Number of labor-units deaths (this tick)
number-of-labor-units-deaths-this-tick
17
1
11

MONITOR
13
712
163
757
Social fractionalization (ELF)
social-fractionalization
2
1
11

MONITOR
460
712
610
757
Cultural polarization (RQ)
cultural-polarization
2
1
11

MONITOR
162
712
312
757
Depth of differences
depth-of-differences
2
1
11

MONITOR
311
712
461
757
Generalized fractionalization
generalized-fractionalization
2
1
11

TEXTBOX
173
691
540
725
FRACTIONALIZATION & POLARIZATION INDICES
14
0.0
1

TEXTBOX
9
765
598
783
- Social fractionalization: measures the probability that two randomly chosen families belong to different groups
11
0.0
1

TEXTBOX
8
778
679
799
- Generalized fractionalization: average expected level of dissimilarity between two families drawn at random from the population
11
0.0
1

SWITCH
872
653
1061
686
print-spatial-matrix?
print-spatial-matrix?
0
1
-1000

SWITCH
872
685
1061
718
display-influence-area?
display-influence-area?
0
1
-1000

TEXTBOX
709
692
804
710
SPATIAL INDEX
14
0.0
1

MONITOR
650
712
733
757
NIL
spatial-index
3
1
11

MONITOR
734
712
848
757
NIL
aggregation-index
2
1
11

MONITOR
670
758
848
803
NIL
adjusted-aggregation-index
17
1
11

@#$#@#$#@
## WHAT IS IT?

(a general understanding of what the model is trying to show or explain)

## HOW IT WORKS

(what rules the agents use to create the overall behavior of the model)

## HOW TO USE IT

(how to use the model, including a description of each of the items in the Interface tab)

## THINGS TO NOTICE

(suggested things for the user to notice while running the model)

## THINGS TO TRY

(suggested things for the user to try to do (move sliders, switches, etc.) with the model)

## EXTENDING THE MODEL

(suggested things to add or change in the Code tab to make the model more complicated, detailed, accurate, etc.)

## NETLOGO FEATURES

(interesting or unusual features of NetLogo that the model uses, particularly in the Code tab; or where workarounds were needed for missing features)

## RELATED MODELS

(models in the NetLogo Models Library and elsewhere which are of related interest)

## CREDITS AND REFERENCES

(a reference to the model's URL on the web if it has one, as well as any other necessary credits, citations, and links)
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

sheep
false
15
Circle -1 true true 203 65 88
Circle -1 true true 70 65 162
Circle -1 true true 150 105 120
Polygon -7500403 true false 218 120 240 165 255 165 278 120
Circle -7500403 true false 214 72 67
Rectangle -1 true true 164 223 179 298
Polygon -1 true true 45 285 30 285 30 240 15 195 45 210
Circle -1 true true 3 83 150
Rectangle -1 true true 65 221 80 296
Polygon -1 true true 195 285 210 285 210 240 240 210 195 210
Polygon -7500403 true false 276 85 285 105 302 99 294 83
Polygon -7500403 true false 219 85 210 105 193 99 201 83

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

wolf
false
0
Polygon -16777216 true false 253 133 245 131 245 133
Polygon -7500403 true true 2 194 13 197 30 191 38 193 38 205 20 226 20 257 27 265 38 266 40 260 31 253 31 230 60 206 68 198 75 209 66 228 65 243 82 261 84 268 100 267 103 261 77 239 79 231 100 207 98 196 119 201 143 202 160 195 166 210 172 213 173 238 167 251 160 248 154 265 169 264 178 247 186 240 198 260 200 271 217 271 219 262 207 258 195 230 192 198 210 184 227 164 242 144 259 145 284 151 277 141 293 140 299 134 297 127 273 119 270 105
Polygon -7500403 true true -1 195 14 180 36 166 40 153 53 140 82 131 134 133 159 126 188 115 227 108 236 102 238 98 268 86 269 92 281 87 269 103 269 113

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 5.3
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
<experiments>
  <experiment name="experiment" repetitions="30" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>go</go>
    <final>file-close</final>
    <timeLimit steps="500"/>
    <enumeratedValueSet variable="cooperation-allowed?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="help-limited-to-spare-labor?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="initial-population">
      <value value="10"/>
      <value value="100"/>
      <value value="500"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="mean-resource-on-patches">
      <value value="4000"/>
      <value value="10000"/>
      <value value="20000"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="average-technology">
      <value value="0.05"/>
      <value value="0.5"/>
      <value value="1"/>
      <value value="2"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="diversity">
      <value value="0"/>
      <value value="0.5"/>
      <value value="0.9"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="labor-average">
      <value value="3"/>
      <value value="5"/>
      <value value="8"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="average-storing-factor">
      <value value="0"/>
      <value value="0.5"/>
      <value value="0.9"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="movement">
      <value value="1"/>
      <value value="5"/>
      <value value="10"/>
      <value value="80"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="internal-change-rate">
      <value value="0.01"/>
      <value value="0.5"/>
      <value value="0.9"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="display-resorces?">
      <value value="false"/>
    </enumeratedValueSet>
  </experiment>
</experiments>
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 0.0 1.0
0.0 1 1.0 0.0
0.2 0 0.0 1.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180

@#$#@#$#@
0
@#$#@#$#@
